# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: transport.proto

from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor.FileDescriptor(
  name='transport.proto',
  package='transport',
  syntax='proto2',
  serialized_options=None,
  serialized_pb=b'\n\x0ftransport.proto\x12\ttransport\"\x1d\n\x08RawField\x12\x11\n\traw_bytes\x18\x01 \x02(\x0c'
)




_RAWFIELD = _descriptor.Descriptor(
  name='RawField',
  full_name='transport.RawField',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='raw_bytes', full_name='transport.RawField.raw_bytes', index=0,
      number=1, type=12, cpp_type=9, label=2,
      has_default_value=False, default_value=b"",
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=30,
  serialized_end=59,
)

DESCRIPTOR.message_types_by_name['RawField'] = _RAWFIELD
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

RawField = _reflection.GeneratedProtocolMessageType('RawField', (_message.Message,), {
  'DESCRIPTOR' : _RAWFIELD,
  '__module__' : 'transport_pb2'
  # @@protoc_insertion_point(class_scope:transport.RawField)
  })
_sym_db.RegisterMessage(RawField)


# @@protoc_insertion_point(module_scope)
